#!/bin/sh
set -eu

PROGRAM=${0##*/}

# defaults
: ${PACKAGE_NAME=''}
: ${NETWORKING=false}
: ${TRACING=false}
: ${WITH=}
: ${WITHOUT=}
: ${KEEP_CONTAINER=false}
: ${TMPFS="4G"}

dir=$(pwd)
image=registry.gitlab.com/pld-linux/cleanbuild
topdir=$dir/rpm
home=/home/builder

die() {
	echo >&2 "$0: $*"
	exit 1
}

is_no() {
	# Test syntax
	if [ $# = 0 ]; then
		return 2
	fi

	case "$1" in
	no|No|NO|false|False|FALSE|off|Off|OFF|N|n|0)
		# true returns zero
		return 0
		;;
	*)
		# false returns one
		return 1
		;;
	esac
}

tmpfs() {
	if is_no "${TMPFS:-true}" || [ "$TMPFS" = "0" ]; then
		return
	fi

	echo "--tmpfs $home/rpm/BUILD:rw,exec,nosuid,size=$TMPFS"
}

create_container() {
	# cleanup first
	docker kill $name >/dev/null 2>&1 || :
	docker rm $name >/dev/null 2>&1 || :

	install -d $topdir/logs

	# start the container
	docker run --name=$name -d \
		-w $home \
		-v $topdir:$home/rpm \
		-v $dir:$home/cleanbuild \
		-v $dir/cache/poldek:/var/cache/poldek \
		-v $dir/cache/ccache/$PACKAGE_NAME:$home/.ccache \
		--label=cleanbuild=$name \
		$(tmpfs) \
		$image >/dev/null

	# set the homedir
	docker exec --user=root -w / $name usermod -d $home builder

	# these paths need to be accessible for builder
	docker exec --user=root -w / $name sh -c "cd $home && chown builder:builder rpm rpm/logs rpm/BUILD .ccache"

	if [ ! -d $topdir/rpm-build-tools ]; then
		docker exec $name builder --init-rpm-dir
	fi
}

package_prepare() {
	# fetch sources and install deps
	if [ -d $topdir/packages/$PACKAGE_NAME ]; then
		# chown, as it might be different owner (root) modified outside container
		docker exec --user=root -w / $name chown -R builder:builder $home/rpm/packages/$PACKAGE_NAME
	fi
	docker exec $name builder -g $PACKAGE_NAME

	# prevent network access like pld builders do
	$NETWORKING || docker exec --user=root -w / $name setfacl -m u:builder:--- /etc/resolv.conf

	git_tag=$(GIT_DIR=$topdir/packages/$PACKAGE_NAME/.git git describe --tags --always)
	buildlog=rpm/logs/${git_tag#auto/*/}.log
}

package_build() {
	# create default args for builder
	set -- -nn ${WITH:+--with "${WITH# }"} ${WITHOUT:+--without "${WITHOUT# }"} "$PACKAGE_NAME"

	while true; do
		# install deps
		docker exec $name builder -g -R "$@"
		# remove .la dependencies
		docker exec --user=root -w / $name $home/cleanbuild/bin/cleanup-la
		# reset findunusedbr state after deps install
		docker exec --user=root -w / $name $home/cleanbuild/bin/findunusedbr -c / $home/rpm/packages/$PACKAGE_NAME/$PACKAGE_NAME.spec

		# actual build
		docker exec $name cleanbuild/bin/teeboth $buildlog builder -bb --define '__spec_clean_body %{nil}' "$@" && rc=$? || rc=$?

		findbr=$PACKAGE_NAME.findbr.log
		builddir=$(docker exec $name sh -c 'test ! -d rpm/BUILD/* || echo rpm/BUILD/*')
		if [ -z "$builddir" ]; then
			echo >&2 "No build dir. Build failed?"
			exit 6
		fi
		# need root to run poldek
		docker exec --user=root -w / $name sh -c "cd $home && cleanbuild/bin/findbr $builddir $buildlog" > $findbr

		installed_something=false
		while read pkg msg; do
			./addbr rpm/packages/$PACKAGE_NAME/$PACKAGE_NAME.spec "$pkg" "$msg" || continue
			installed_something=true
		done < $findbr
		rm -f $findbr

		# go for another try
		$installed_something && continue

		docker exec --user=root -w / $name $home/cleanbuild/bin/findunusedbr -c / $home/rpm/packages/$PACKAGE_NAME/$PACKAGE_NAME.spec

		if [ $rc -eq 0 ] && ! $KEEP_CONTAINER; then
			# finished ok, cleanup
			docker kill $name >/dev/null && docker rm $name >/dev/null || :
		fi

		# propagate error
		exit $rc
	done
}

parse_options() {
	local t
	t=$(getopt -o 'x' --long 'network,no-tmpfs,notmpfs,tmpfs:,keep-container,with:,without:' -n "$PROGRAM" -- "$@")
	[ $? != 0 ] && exit $?
	eval set -- "$t"

	while :; do
		case "$1" in
		-x)
			TRACING=true
			;;
		--network)
			NETWORKING=true
			;;
		--no-tmpfs|--notmpfs)
			TMPFS=false
			;;
		--tmpfs)
			shift
			TMPFS="$1"
			;;
		--keep-container)
			KEEP_CONTAINER=true
			;;
		--with)
			shift
			WITH="$WITH $1"
			;;
		--without)
			shift
			WITHOUT="$WITHOUT $1"
			;;
		--)
			shift
			break
			;;
		*)
			die "Internal error: [$1] not recognized!"
			;;
		esac
		shift
	done

	test "$#" -eq 1 || die "package not specified or excess arguments"
	PACKAGE_NAME="${1%.spec}"
}

main() {
	parse_options "$@"

	$TRACING && set -x
	local name="cleanbuild-$PACKAGE_NAME"
	create_container
	package_prepare
	package_build
}

main "$@"
